@@ -18,6 +18,7 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <config.h>
 #include "gtkemuvt102.h"
 #include <stdio.h>
 
@@ -156,10 +157,10 @@ gpointer
 gtk_emu_vt102_new (void)
 {
   GtkTtyEmuData *emu_data;
-
+  
   emu_data = g_new (GtkTtyEmuData, 1);
   *emu_data = emu_data_default;
-
+  
   return emu_data;
 }
 
@@ -185,41 +186,44 @@ gtk_emu_vt102_reset (GtkTty	*tty,
   gtk_term_reset (GTK_TERM (tty));
   if (blank_screen)
     gtk_term_clear (GTK_TERM (tty), TRUE, TRUE);
-
+  
   return VT_TERM;
 }
 
 guint
 gtk_emu_vt102 (GtkTty	      *tty,
-	     gpointer	    write_func_data,
-	     guchar	    *buffer,
-	     guint	    count)
+	       gpointer	    write_func_data,
+	       guchar	    *buffer,
+	       guint	    count)
 {
   /* this function is modeled much after
    * linux-2.0.29/drivers/char/console.c:con_write().
    * for the moment we skip unicode and charset conversions
    */
-  
   GtkTerm *term;
   guint n_processed;
   GtkTtyEmuData *emu;
   guint old_x, old_y;
-
+  
   term = GTK_TERM (tty);
   emu = write_func_data;
-
+  
+  /*
+   * printf("buffer_count: %d\n", count);
+   */
+  
   gtk_term_set_cursor_mode (term, GTK_CURSOR_INVISIBLE, emu->cursor_blinking);
-
+  
   gtk_term_get_cursor (term, &emu->cur_x, &emu->cur_y);
   old_x = emu->cur_x;
   old_y = emu->cur_y;
-
+  
   n_processed = 0;
   while (n_processed < count)
   {
     guchar ch;
     gboolean display_ok;
-
+    
     if (emu->cur_x != old_x ||
 	emu->cur_y != old_y)
     {
@@ -227,7 +231,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
       old_x = emu->cur_x;
       old_y = emu->cur_y;
     }
-
+    
     ch = buffer[n_processed++];
     display_ok = (ch
 		  && (ch >= 32 ||
@@ -247,28 +251,28 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	gtk_emu_vt102_lf (tty, emu);
       }
 
-      /* FIXME: autowrap hint */
+      gtk_term_set_scroll_offset (term, 0);
       emu->need_wrap = gtk_term_putc (term, ch, emu->insert_mode);
       gtk_term_get_cursor (term, &emu->cur_x, &emu->cur_y);
-
+      
       continue;
     }
-
+    
     /* control characters can be used in the middle of an escape sequence
      */
     switch (ch)
     {
     case  0:
       continue;
-
+      
     case  '\a': /* audible bell */
       gtk_term_bell (term);
       continue;
-
+      
     case  '\b': /* backspace */
       gtk_emu_vt102_bs (tty, emu);
       continue;
-
+      
     case  '\t': /* tabulator */
       while (emu->cur_x < MIN (160, term->term_width) - 1)
       {
@@ -281,7 +285,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
       }
       gtk_term_set_cursor (term, emu->cur_x, emu->cur_y);
       continue;
-
+      
     case  '\n': /* linefeed */
     case  '\v': /* vertical tabulator */
     case  '\f': /* formfeed */
@@ -289,33 +293,33 @@ gtk_emu_vt102 (GtkTty	      *tty,
       if (emu->cr_and_lf)
 	gtk_emu_vt102_cr (tty, emu);
       continue;
-
+      
     case  '\r': /* carriage return */
       gtk_emu_vt102_cr (tty, emu);
       continue;
-
+      
     case  14: /* shift out */
       /* FIXME: charset */
       continue;
-
+      
     case  15: /* shift in */
       /* FIXME: charset */
       continue;
-
+      
     case  24: /* cancel */
     case  26: /* SUB */
       emu->state = ESC_Normal;
       continue;
-
+      
     case  27: /* escape */
       emu->state = ESC_Escape;
       continue;
-
+      
     case  128 + 27: /* ??? */
       emu->state = ESC_Square;
       continue;
     }
-
+    
     /* handle various escape states
      */
     switch (emu->state)
@@ -393,7 +397,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	continue;
       }
       continue;
-
+      
     case  ESC_NonStd:
       if (ch == 'P') /* palette escape sequence */
       {
@@ -408,7 +412,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
       else
 	emu->state = ESC_Normal;
       continue;
-
+      
     case ESC_Palette:
       if ((ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'F') || (ch >= 'a' &&ch <= 'f'))
       {
@@ -419,7 +423,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
       else
 	emu->state = ESC_Normal;
       continue;
-
+      
     case ESC_Square:
       for (emu->n_params = 0; emu->n_params < MAX_PARAMS; emu->n_params++)
 	emu->param[emu->n_params] = 0;
@@ -451,15 +455,15 @@ gtk_emu_vt102 (GtkTty	      *tty,
       emu->state = ESC_Normal;
       switch (ch)
       {
-
+	
       case  'h':
 	gtk_emu_vt102_set_mode (tty, emu, TRUE);
 	continue;
-
+	
       case 'l':
 	gtk_emu_vt102_set_mode (tty, emu, FALSE);
 	continue;
-
+	
       case 'n':
 	if (!emu->dec_priv_mode)
 	  if (emu->param[0] == 5)
@@ -467,7 +471,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	  else if (emu->param[0] == 6)
 	  {
 	    guchar string[40];
-
+	    
 	    sprintf (string,
 		     "\033[%d;%dR",
 		     emu->cur_y + (emu->relative_origin ? term->top + 1 : 1),
@@ -483,14 +487,14 @@ gtk_emu_vt102 (GtkTty	      *tty,
       }
       switch(ch)
       {
-
+	
       case  'G':
       case  '`':
 	if (emu->param[0])
 	  emu->param[0]--;
 	gtk_emu_vt102_gotoxy (tty, emu, emu->param[0], emu->cur_y);
 	continue;
-
+	
       case  'A':
 	if (!emu->param[0])
 	  emu->param[0] = 1;
@@ -498,21 +502,21 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	  emu->param[0] = emu->cur_y;
 	gtk_emu_vt102_gotoxy (tty, emu, emu->cur_x, emu->cur_y - emu->param[0]);
 	continue;
-
+	
       case  'B':
       case  'e':
 	if (!emu->param[0])
 	  emu->param[0] = 1;
 	gtk_emu_vt102_gotoxy (tty, emu, emu->cur_x, emu->cur_y + emu->param[0]);
 	continue;
-
+	
       case  'C':
       case  'a':
 	if (!emu->param[0])
 	  emu->param[0] = 1;
 	gtk_emu_vt102_gotoxy (tty, emu, emu->cur_x + emu->param[0], emu->cur_y);
 	continue;
-
+	
       case  'D':
 	if (!emu->param[0])
 	  emu->param[0] = 1;
@@ -520,7 +524,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	  emu->param[0] = emu->cur_x;
 	gtk_emu_vt102_gotoxy (tty, emu, emu->cur_x - emu->param[0], emu->cur_y);
 	continue;
-
+	
       case  'E':
 	if (!emu->param[0])
 	  emu->param[0] = 1;
@@ -585,7 +589,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	if (!emu->param[0])
 	  gtk_tty_put_in (tty, VT102ID, sizeof (VT102ID) - 1);
 	continue;
-
+	
       case  'g':
 	if (!emu->param[0])
 	  emu->tab_stop[MIN (emu->cur_x, 160) >> 5] &= ~(1 << (MIN (emu->cur_x, 160) & 31));
@@ -598,11 +602,11 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	  emu->tab_stop[4] = 0;
 	}
 	continue;
-
+	
       case  'm':
 	gtk_emu_vt102_set_term_mode (tty, emu);
 	continue;
-
+	
       case  'q':
 	if (emu->param[0] < 8)
 	{
@@ -614,7 +618,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	  gtk_tty_leds_changed (tty);
 	}
 	continue;
-
+	
       case  'r':
 	if (!emu->param[0])
 	  emu->param[0] = 1;
@@ -627,44 +631,44 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	  gtk_emu_vt102_gotoxry (tty, emu, 0, 0);
 	}
 	continue;
-
+	
       case  's':
 	gtk_term_save_cursor (term);
 	continue;
-
+	
       case  'u':
 	gtk_term_restore_cursor (term);
 	gtk_term_get_cursor (term, &emu->cur_x, &emu->cur_y);
 	emu->need_wrap = FALSE;
 	continue;
-
+	
       case  'X':
 	gtk_term_erase_chars (term, emu->param[0]);
 	emu->need_wrap = FALSE;
 	continue;
-
+	
       case  '@':
 	gtk_term_insert_chars (term, emu->param[0]);
 	continue;
-
+	
       case  ']':
 	gtk_emu_vt102_linux_setterm (tty, emu);
 	continue;
       }
       continue;
-
+      
     case  ESC_Percent:
       emu->state = ESC_Normal;
       continue;
-
+      
     case  ESC_FuncKey:
       emu->state = ESC_Normal;
       continue;
-
+      
     case  ESC_Hash:
       emu->state = ESC_Normal;
       continue;
-
+      
     case  ESC_Set_G0:
       emu->state = ESC_Normal;
       if (ch == '0')
@@ -684,7 +688,7 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	/* FIXME: charset_G0 = USER_MAP */
       }
       continue;
-
+      
     case  ESC_Set_G1:
       emu->state = ESC_Normal;
       if (ch == '0')
@@ -704,17 +708,17 @@ gtk_emu_vt102 (GtkTty	      *tty,
 	/* FIXME: charset_G1 = USER_MAP */
       }
       continue;
-
+      
     case  ESC_Normal:
       continue;
-
+      
     default:
       emu->state = ESC_Normal;
     }
   }
-
+  
   gtk_term_set_cursor_mode (term, emu->cursor_mode, emu->cursor_blinking);
-
+  
   return n_processed;
 }
 
