@@ -18,6 +18,7 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <config.h>
 #include "gtkterm.h"
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
@@ -30,7 +31,7 @@
 #define	BLINK_TIMEOUT			(333)
 #define	MIN_WIDTH			(15)
 #define	MAX_WIDTH			(1024)
-#define	MIN_HEIGHT			(2)
+#define	MIN_HEIGHT			(1)
 #define	MAX_HEIGHT			(1024)
 #define	UNDERLINE_THIKNESS		(1)
 #define	CURSOR_THIKNESS			(3)
@@ -47,7 +48,8 @@ enum
   FLAG_BOLD		= 1 << 1,
   FLAG_UNDERLINE	= 1 << 2,
   FLAG_REVERSE		= 1 << 3,
-  FLAG_SELECTION	= 1 << 4
+  FLAG_SELECTION	= 1 << 4,
+  FLAG_DIRTY		= 1 << 5
 };
 
 enum
@@ -264,6 +266,8 @@ gtk_term_init (GtkTerm *term)
   term->term_width = 0;
   term->term_height = 0;
   term->first_line = 0;
+  term->first_used_line = 0;
+  term->scroll_offset = 0;
   term->term_inversed = FALSE;
   
   term->dim = FALSE;
@@ -272,7 +276,7 @@ gtk_term_init (GtkTerm *term)
   term->reverse = FALSE;
   term->i_fore = GTK_TERM_MAX_COLORS - 1;
   term->i_back = 0;
-
+  
   term->cursor_mode = DEFAULT_CURSOR_MODE;
   term->cursor_blinking = DEFAULT_CURSOR_BLINK;
   term->cur_x = 0;
@@ -363,7 +367,8 @@ gtk_term_setup (GtkTerm	      *term,
   term->term_height = MIN (height, term->max_term_height);
   term->first_line = term->max_term_height - term->term_height;
   term->first_used_line = term->first_line;
-
+  term->scroll_offset = 0;
+  
   term->cur_y = term->first_line;
   
   for (i = 0; i < GTK_TERM_MAX_COLORS; i++)
@@ -384,7 +389,7 @@ gtk_term_setup (GtkTerm	      *term,
     
     gtk_term_line_init (term, i);
   }
-
+  
   gtk_term_reset (term);
 }
 
@@ -451,6 +456,7 @@ gtk_term_realize (GtkWidget *widget)
   gdk_window_set_user_data (term->text_area, term);
   gdk_window_set_background (term->text_area, term->back[0]);
   gdk_window_show (term->text_area);
+  term->scroll_offset = 0;
   
   term->text_gc = gdk_gc_new (term->text_area);
   gdk_gc_set_exposures (term->text_gc, TRUE);
@@ -553,9 +559,9 @@ gtk_term_size_allocate (GtkWidget     *widget,
   
   gtk_signal_emit (GTK_OBJECT (term), term_signals[TEXT_RESIZE],
 		   &new_width, &new_height);
-
+  
   SEL_MAKE_VOID (term);
-
+  
   cursor_x_saved = term->cur_x;
   cursor_y_inv_saved = term->max_term_height - term->cur_y;
   term->cur_x = 0;
@@ -589,19 +595,44 @@ gtk_term_size_allocate (GtkWidget     *widget,
 			    1 + widget->style->klass->ythickness + INNER_BORDER,
 			    term->term_width * term->char_width,
 			    term->term_height * term->char_height);
-    gdk_window_move (term->text_area,
-		     0,
-		     - term->first_line * term->char_height);
     
     term->cur_x = MIN (cursor_x_saved, term->term_width - 1);
     term->cur_y = MAX (term->max_term_height - cursor_y_inv_saved, term->first_line);
-  }
 
+    gtk_term_set_scroll_offset (term, 0);
+  }
+  
   gtk_term_reset (term);
   if (GTK_WIDGET_DRAWABLE (widget))
     gtk_term_update_cursor (term);
 }
 
+gint
+gtk_term_set_scroll_offset (GtkTerm	*term,
+			    gint	offset)
+{
+  gint lower_bound;
+  
+  g_return_val_if_fail (term != NULL, 0);
+  g_return_val_if_fail (GTK_IS_TERM (term), 0);
+  
+  lower_bound = - (term->first_line - term->first_used_line);
+  
+  if (offset < lower_bound)
+    term->scroll_offset = lower_bound;
+  else if (offset > 0)
+    term->scroll_offset = 0;
+  else
+    term->scroll_offset = offset;
+
+  if (GTK_WIDGET_REALIZED (term))
+    gdk_window_move (term->text_area,
+		     0,
+		     - (term->first_line + term->scroll_offset) * term->char_height);
+  
+  return term->scroll_offset;
+}
+
 static void
 gtk_term_draw_focus (GtkWidget *widget)
 {
@@ -656,7 +687,7 @@ gtk_term_draw (GtkWidget    *widget,
   if (GTK_WIDGET_DRAWABLE (widget))
   {
     gtk_widget_draw_focus (widget);
-    gtk_term_draw_area (term, NULL);
+    gtk_term_update_area (term, NULL);
   }
 }
 
@@ -675,7 +706,7 @@ gtk_term_expose (GtkWidget	*widget,
   if (widget->window == event->window)
     gtk_widget_draw_focus (widget);
   else if (term->text_area == event->window)
-    gtk_term_draw_area (term, &event->area);
+    gtk_term_update_area (term, &event->area);
   
   return FALSE;
 }
@@ -783,7 +814,7 @@ gtk_term_button_release (GtkWidget	*widget,
   if (event->button == 1 || event->button == 3)
   {
     gtk_grab_remove (widget);
-
+    
     if (term->sel_valid)
       gtk_selection_owner_set (GTK_WIDGET (term), GDK_SELECTION_PRIMARY, event->time);
   }
@@ -927,19 +958,19 @@ gtk_term_selection_clear (GtkWidget		*widget,
   /* g_return_val_if_fail (event != NULL, FALSE); */
   
   term = GTK_TERM (widget);
-
+  
   if (term->sel_buffer)
   {
     g_free (term->sel_buffer);
     term->sel_buffer = NULL;
     term->sel_len = 0;
   }
-
+  
   if (term->sel_valid)
   {
     gtk_selection_owner_set (NULL, GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME);
     term->sel_valid = FALSE;
-
+    
     if (GTK_WIDGET_DRAWABLE (widget))
       gtk_term_update_new_sel (term,
 			       term->sel_b_x,
@@ -949,7 +980,7 @@ gtk_term_selection_clear (GtkWidget		*widget,
 			       term->sel_b_x,
 			       term->sel_b_y);
   }
-
+  
   return FALSE;
 }
 
@@ -996,7 +1027,7 @@ gtk_entry_selection_handler (GtkWidget		*widget,
     for (; y <= e_y; y++)
     {
       guint e, b;
-
+      
       if (y == e_y)
 	e = e_x;
       else
@@ -1005,12 +1036,12 @@ gtk_entry_selection_handler (GtkWidget		*widget,
 	     (term->char_buffer[y][e] == ' ' ||
 	      term->char_buffer[y][e] == '\t'))
 	e--;
-
+      
       if (y == b_y)
 	b = b_x;
       else
 	b = 0;
-
+      
       if (e == 0 &&
 	  (term->char_buffer[y][e] == ' ' ||
 	   term->char_buffer[y][e] == '\t'))
@@ -1026,13 +1057,13 @@ gtk_entry_selection_handler (GtkWidget		*widget,
 	  e_x >= term->term_width - 1)
 	sel_buffer[i++] = '\n';
     }
-
+    
     sel_buffer[i] = 0;
     term->sel_buffer = g_strdup (sel_buffer);
     term->sel_len = i;
     g_free (sel_buffer);
   }
-
+  
   if (term->sel_buffer)
   {
     gtk_selection_data_set (selection_data,
@@ -1122,7 +1153,7 @@ gtk_term_destroy (GtkObject *object)
   term->attrib_buffer = NULL;
   g_free (term->char_buffer);
   term->char_buffer = NULL;
-
+  
   if (GTK_OBJECT_CLASS (parent_class)->destroy)
     (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -1136,7 +1167,7 @@ gtk_term_set_scroll_reg (GtkTerm	*term,
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->top = MIN (top, term->term_height - 2);
   term->bottom = CLAMP (bottom, term->top + 1, term->term_height - 1);
 }
@@ -1149,9 +1180,9 @@ gtk_term_scroll (GtkTerm	*term,
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
   NEW_INPUT (term);
-
+  
   n = CLAMP (n, 1, term->bottom - term->top + 1);
-
+  
   if (downwards)
     gtk_term_scroll_down (term, term->first_line + term->top, term->first_line + term->bottom, n);
   else
@@ -1165,9 +1196,9 @@ gtk_term_delete_lines (GtkTerm	*term,
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
   NEW_INPUT (term);
-
+  
   n = MIN (n ? n : 1, term->max_term_height - term->cur_y);
-
+  
   gtk_term_scroll_up (term, term->cur_y, term->first_line + term->bottom, n);
 }
 
@@ -1177,31 +1208,31 @@ void gtk_term_insert_lines (GtkTerm	*term,
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
   NEW_INPUT (term);
-
+  
   n = MIN (n ? n : 1, term->max_term_height - term->cur_y);
-
+  
   gtk_term_scroll_down (term, term->cur_y, term->first_line + term->bottom, n);
 }
 
 void
-gtk_term_set_color (GtkTerm        *term,
-		    guint          index,
-		    GdkColor       *back,
-		    GdkColor       *fore,
-		    GdkColor       *fore_dim,
-		    GdkColor       *fore_bold)
+gtk_term_set_color (GtkTerm	   *term,
+		    guint	   index,
+		    GdkColor	   *back,
+		    GdkColor	   *fore,
+		    GdkColor	   *fore_dim,
+		    GdkColor	   *fore_bold)
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
   g_return_if_fail (index < GTK_TERM_MAX_COLORS);
   g_return_if_fail (back != NULL);
   g_return_if_fail (fore != NULL);
-
+  
   if (!fore_dim)
     fore_dim = fore;
   if (!fore_bold)
     fore_bold = fore;
-
+  
   term->back[index] = back;
   term->fore[index] = fore;
   term->fore_dim[index] = fore_dim;
@@ -1271,7 +1302,7 @@ gtk_term_save_cursor (GtkTerm	     *term)
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->s_cur_x = term->cur_x;
   term->s_cur_y = term->cur_y;
   term->s_text_mode = ((term->dim	? FLAG_DIM	 : 0) |
@@ -1287,7 +1318,7 @@ gtk_term_restore_cursor (GtkTerm	*term)
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->cur_x = term->s_cur_x;
   term->cur_y = term->s_cur_y;
   term->dim = term->s_text_mode & FLAG_DIM;
@@ -1387,7 +1418,7 @@ gtk_term_delete_chars (GtkTerm	      *term,
       term->char_buffer[term->cur_y][term->cur_x + i] = term->char_buffer[term->cur_y][term->cur_x + gap + i];
       term->attrib_buffer[term->cur_y][term->cur_x + i] = term->attrib_buffer[term->cur_y][term->cur_x + gap + i];
     }
-
+    
     for (i = term->term_width - 1; i >= term->term_width - gap; i--)
     {
       term->char_buffer[term->cur_y][i] = gtk_term_blank_char;
@@ -1428,7 +1459,7 @@ gtk_term_insert_chars (GtkTerm	*term,
   CURSOR_OFF (term);
   
   n = MIN (n ? n : 1, term->term_width - term->cur_x);
-
+  
   for (i = term->term_width - 1; i >= term->cur_x + n; i--)
   {
     term->char_buffer[term->cur_y][i] = term->char_buffer[term->cur_y][i - n];
@@ -1443,10 +1474,10 @@ gtk_term_insert_chars (GtkTerm	*term,
   
   if (GTK_WIDGET_DRAWABLE (term))
   {
-    gtk_term_draw_line (term,
-			term->cur_y,
-			term->cur_x + n,
-			term->term_width);
+    gtk_term_update_line (term,
+			  term->cur_y,
+			  term->cur_x + n,
+			  term->term_width);
     gdk_window_clear_area (term->text_area,
 			   term->cur_x * term->char_width,
 			   term->cur_y * term->char_height,
@@ -1466,14 +1497,14 @@ gtk_term_clear_line (GtkTerm	*term,
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
   NEW_INPUT (term);
-
+  
   if (before_cursor)
     for (i = 0; i < term->cur_x; i++)
     {
       term->char_buffer[term->cur_y][i] = gtk_term_blank_char;
       term->attrib_buffer[term->cur_y][i] = gtk_term_blank_attrib;
     }
-
+  
   if (after_cursor)
     for (i = term->cur_x + 1; i < term->term_width; i++)
     {
@@ -1483,11 +1514,11 @@ gtk_term_clear_line (GtkTerm	*term,
   
   term->char_buffer[term->cur_y][term->cur_x] = gtk_term_blank_char;
   term->attrib_buffer[term->cur_y][term->cur_x] = gtk_term_blank_attrib;
-
+  
   if (GTK_WIDGET_DRAWABLE (term))
   {
     guint b, e;
-
+    
     b = before_cursor ? 0 : term->cur_x;
     e = after_cursor ? term->term_width - 1 : term->cur_x;
     gdk_window_clear_area (term->text_area,
@@ -1550,10 +1581,10 @@ gtk_term_putc (GtkTerm	*term,
   g_return_val_if_fail (GTK_IS_TERM (term), FALSE);
   NEW_INPUT (term);
   CURSOR_OFF (term);
-
+  
   if (insert)
     gtk_term_insert_chars (term, 1);
-
+  
   term->char_buffer[term->cur_y][term->cur_x] = ch;
   term->attrib_buffer[term->cur_y][term->cur_x].i_fore = term->i_fore;
   term->attrib_buffer[term->cur_y][term->cur_x].i_back = term->i_back;
@@ -1562,7 +1593,7 @@ gtk_term_putc (GtkTerm	*term,
   term->attrib_buffer[term->cur_y][term->cur_x].flags |= term->bold ? FLAG_BOLD : 0;
   term->attrib_buffer[term->cur_y][term->cur_x].flags |= term->underline ? FLAG_UNDERLINE : 0;
   term->attrib_buffer[term->cur_y][term->cur_x].flags |= term->reverse ? FLAG_REVERSE : 0;
-
+  
   if (GTK_WIDGET_DRAWABLE (term))
     gtk_term_update_char (term, 
 			  term->cur_x,
@@ -1576,9 +1607,9 @@ gtk_term_putc (GtkTerm	*term,
   }
   else
     need_wrap = FALSE;
-
+  
   CURSOR_ON (term);
-
+  
   return need_wrap;
 }
 
@@ -1586,7 +1617,7 @@ void
 gtk_term_bell (GtkTerm	      *term)
 {
   /* FIXME: invoke signal */
-
+  
   gdk_beep ();
 }
 
@@ -1595,20 +1626,20 @@ gtk_term_reset (GtkTerm *term)
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->dim = FALSE;
   term->bold = FALSE;
   term->underline = FALSE;
   term->reverse = FALSE;
   term->i_fore = GTK_TERM_MAX_COLORS - 1;
   term->i_back = 0;
-
+  
   term->cursor_mode = DEFAULT_CURSOR_MODE;
   term->cursor_blinking = DEFAULT_CURSOR_BLINK;
-
+  
   term->top = 0;
   term->bottom = term->term_height - 1;
-
+  
   gtk_term_save_cursor (term);
 }
 
@@ -1618,7 +1649,7 @@ gtk_term_set_dim (GtkTerm	 *term,
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->dim = dim != FALSE;
 }
 
@@ -1628,37 +1659,37 @@ gtk_term_set_bold (GtkTerm	  *term,
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->bold = bold != FALSE;
 }
 
 void
 gtk_term_set_underline (GtkTerm	       *term,
-			 gboolean	underline)
+			gboolean	underline)
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->underline = underline != FALSE;
 }
-     
+
 void
 gtk_term_set_reverse (GtkTerm	     *term,
 		      gboolean	     reverse)
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->reverse = reverse != FALSE;
 }
-     
+
 void
 gtk_term_set_inversed (GtkTerm	      *term,
 		       gboolean	      term_inversed)
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TERM (term));
-
+  
   term->term_inversed = term_inversed != FALSE;
   /* FIXME: inverse screen */
 }
