@@ -18,6 +18,7 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <config.h>
 #include "gtktty.h"
 #include <unistd.h>
 #include <stdlib.h>
@@ -34,17 +35,25 @@
 
 
 /* --- limits --- */
-#define INPUT_BUFFER_LEN	(4096)
+#define	INPUT_BUFFER_LEN	(8192)
+#define	MAX_SCROLL_BACK		(999)
 
 /* --- typedefs --- */
 enum {
   /* FIXME: led-update */
+  KEY_PRESS,
   PROGRAM_EXEC,
   PROGRAM_EXIT,
   LAST_SIGNAL
 };
 
 
+typedef	gint	(*GtkTtySignalKeyPress)		(GtkObject	*object,
+						 gpointer	arg1,
+						 guint		arg2,
+						 guint		arg3,
+						 guint		arg4,
+						 gpointer	data);
 typedef	void	(*GtkTtySignalExec)		(GtkObject	*object,
 						 gpointer	arg1,
 						 gpointer	arg2,
@@ -58,6 +67,10 @@ typedef	void	(*GtkTtySignalExit)		(GtkObject	*object,
 
 
 /* --- prototypes --- */
+static	void	gtk_tty_marshal_signal_key_press (GtkObject		*object,
+						  GtkSignalFunc	func,
+						  gpointer		func_data,
+						  GtkArg		*args);
 static	void	gtk_tty_marshal_signal_exec	(GtkObject		*object,
 						 GtkSignalFunc		func,
 						 gpointer		func_data,
@@ -147,6 +160,16 @@ gtk_tty_class_init (GtkTtyClass *class)
   if (!parent_class)
     parent_class = gtk_type_class (gtk_term_get_type ());
   
+  tty_signals[KEY_PRESS] = gtk_signal_new ("key_press",
+					   GTK_RUN_LAST,
+					   object_class->type,
+					   GTK_SIGNAL_OFFSET (GtkTtyClass, key_press),
+					   gtk_tty_marshal_signal_key_press,
+					   GTK_TYPE_INT, 4,
+					   GTK_TYPE_STRING,
+					   GTK_TYPE_UINT,
+					   GTK_TYPE_UINT,
+					   GTK_TYPE_UINT);
   tty_signals[PROGRAM_EXEC] = gtk_signal_new ("program_exec",
 					      GTK_RUN_LAST,
 					      object_class->type,
@@ -166,15 +189,16 @@ gtk_tty_class_init (GtkTtyClass *class)
 					      GTK_TYPE_CHAR,
 					      GTK_TYPE_UINT);
   gtk_object_class_add_signals (object_class, tty_signals, LAST_SIGNAL);
-
+  
   object_class->destroy = gtk_tty_destroy;
   
   widget_class->key_press_event = gtk_tty_key_press_event;
   widget_class->key_release_event = gtk_tty_key_release_event;
   
   term_class->text_resize = gtk_tty_text_resize;
-
+  
   class->os_hintp = gtk_tty_os_get_hintp ();
+  class->key_press = NULL;
   class->program_exec = NULL;
   class->program_exit = NULL;
 }
@@ -189,10 +213,10 @@ gtk_tty_init (GtkTty *tty)
   tty->freeze_leds = FALSE;
   tty->leds = tty->key_states;
   tty->update_leds = NULL;
-
+  
   tty->key_pad_enter = FALSE;
   tty->key_pad_char = 0;
-
+  
   tty->pid = 0;
   tty->prg_name = NULL;
   tty->exit_status = 0;
@@ -201,11 +225,11 @@ gtk_tty_init (GtkTty *tty)
   tty->sys_sec = 0;
   tty->user_usec = 0;
   tty->user_sec = 0;
-
+  
   tty->pty_fd = -1;
   tty->tty_name = NULL;
   tty->input_tag = 0;
-
+  
   tty->emu_data = NULL;
   tty->emu_term = NULL;
   tty->emu_func = NULL;
@@ -221,19 +245,21 @@ gtk_tty_init (GtkTty *tty)
 
 GtkWidget*
 gtk_tty_new (guint width,
-	       guint height)
+	     guint height,
+	     guint scrollback)
 {
   GtkTty *tty;
   
   tty = gtk_type_new (gtk_tty_get_type ());
-  gtk_term_setup (GTK_TERM (tty), width, height, width, 20);
-
+  scrollback = MIN (scrollback, MAX_SCROLL_BACK);
+  gtk_term_setup (GTK_TERM (tty), width, height, width, scrollback);
+  
   if (tty->emu_reset_func)
   {
     g_free (tty->emu_term);
     tty->emu_term = g_strdup ((* tty->emu_reset_func) (tty, tty->emu_data, TRUE));
   }
-
+  
   return GTK_WIDGET (tty);
 }
 
@@ -245,6 +271,7 @@ gtk_tty_key_press_event (GtkWidget	*widget,
   guchar buffer[64];
   guint count;
   guchar old_key_states;
+  gint	return_val;
   
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TTY (widget), FALSE);
@@ -252,12 +279,12 @@ gtk_tty_key_press_event (GtkWidget	*widget,
   
   if (!GTK_WIDGET_HAS_FOCUS (widget))
     return FALSE;
-
+  
   tty = GTK_TTY (widget);
-
+  
   memset (buffer, 0, sizeof (buffer));
   count = 0;
-
+  
   /* ok, at this point we still have the old state settings in event->state,
    * therefore we need to override the states in the switch() statement
    */
@@ -279,7 +306,7 @@ gtk_tty_key_press_event (GtkWidget	*widget,
       buffer[count++] = '\e';
     buffer[count++] = event->keyval;
     break;
-
+    
   case	' ' ... '/':
   case	'0' ... '9':
   case	':' ... '?':
@@ -290,7 +317,7 @@ gtk_tty_key_press_event (GtkWidget	*widget,
       buffer[count++] = '\e';
     buffer[count++] = event->keyval;
     break;
-
+    
   case	'@':
   case	'A' ... 'Z':
   case	'[' ... '_':
@@ -301,7 +328,7 @@ gtk_tty_key_press_event (GtkWidget	*widget,
     else
       buffer[count++] = event->keyval;
     break;
-
+    
   case	'a' ... 'z':
     if (event->state & GDK_MOD1_MASK)
       buffer[count++] = '\e';
@@ -310,7 +337,7 @@ gtk_tty_key_press_event (GtkWidget	*widget,
     else
       buffer[count++] = event->keyval;
     break;
-
+    
   case	GDK_KP_0:
   case	GDK_KP_1:
   case	GDK_KP_2:
@@ -373,185 +400,207 @@ gtk_tty_key_press_event (GtkWidget	*widget,
     case  GDK_KP_Enter:		buffer[count++] = '\n'; break;
     }
     break;
-
+    
   case	GDK_Escape:
     buffer[count++] = '\033';
     break;
-
+    
   case	GDK_Return:
     buffer[count++] = '\r';
     break;
-
+    
   case	GDK_BackSpace:
     buffer[count++] = '\177';
     break;
-
+    
   case	GDK_Tab:
     buffer[count++] = '\t';
     break;
-
+    
   case	GDK_KP_Right:
   case	GDK_Right:
     count = sprintf (buffer, "\033[C");
     break;
-
+    
   case	GDK_KP_Left:
   case	GDK_Left:
     count = sprintf (buffer, "\033[D");
     break;
-
+    
   case	GDK_KP_Up:
   case	GDK_Up:
     count = sprintf (buffer, "\033[A");
     break;
-
+    
   case	GDK_KP_Down:
   case	GDK_Down:
     count = sprintf (buffer, "\033[B");
     break;
-
+    
   case	GDK_KP_Insert:
   case	GDK_Insert:
     count = sprintf (buffer, "\033[2~");
     break;
-
+    
   case	GDK_KP_Delete:
   case	GDK_Delete:
     count = sprintf (buffer, "\033[3~");
     break;
-
+    
   case	GDK_KP_Home:
   case	GDK_Home:
     count = sprintf (buffer, "\033[1~");
     break;
-
+    
   case	GDK_KP_End:
   case	GDK_End:
     count = sprintf (buffer, "\033[4~");
     break;
-
+    
   case	GDK_KP_Page_Up:		/* GDK_KP_Prior */
   case	GDK_Page_Up:		/* GDK_Prior */
-    count = sprintf (buffer, "\033[5~");
+    if (tty->key_states & GTK_TTY_STATE_SHIFT)
+      gtk_term_set_scroll_offset (GTK_TERM (tty),
+				  GTK_TERM (tty)->scroll_offset -
+				  GTK_TERM (tty)->term_height / 2);
+    else
+      count = sprintf (buffer, "\033[5~");
     break;
-
+    
   case	GDK_KP_Page_Down:	/* GDK_KP_Next */
   case	GDK_Page_Down:		/* GDK_Next */
-    count = sprintf (buffer, "\033[6~");
+    if (tty->key_states & GTK_TTY_STATE_SHIFT)
+      gtk_term_set_scroll_offset (GTK_TERM (tty),
+				  GTK_TERM (tty)->scroll_offset +
+				  GTK_TERM (tty)->term_height / 2);
+    else
+      count = sprintf (buffer, "\033[6~");
     break;
-
+    
   case	GDK_KP_F1:
   case	GDK_F1:
     count = sprintf (buffer, "\033[[A");
     break;
-
+    
   case	GDK_KP_F2:
   case	GDK_F2:
     count = sprintf (buffer, "\033[[B");
     break;
-
+    
   case	GDK_KP_F3:
   case	GDK_F3:
     count = sprintf (buffer, "\033[[C");
     break;
-
+    
   case	GDK_KP_F4:
   case	GDK_F4:
     count = sprintf (buffer, "\033[[D");
     break;
-
+    
   case	GDK_F5:
     count = sprintf (buffer, "\033[[E");
     break;
-
+    
   case	GDK_F6:
     count = sprintf (buffer, "\033[17~");
     break;
-
+    
   case	GDK_F7:
     count = sprintf (buffer, "\033[18~");
     break;
-
+    
   case	GDK_F8:
     count = sprintf (buffer, "\033[19~");
     break;
-
+    
   case	GDK_F9:
     count = sprintf (buffer, "\033[20~");
     break;
-
+    
   case	GDK_F10:
     count = sprintf (buffer, "\033[21~");
     break;
-
+    
   case	GDK_F11:
     count = sprintf (buffer, "\033[23~");
     break;
-
+    
   case	GDK_F12:
     count = sprintf (buffer, "\033[24~");
     break;
-
+    
   case	GDK_Num_Lock:
     if (!(old_key_states & GTK_TTY_STATE_NUM_LOCK))
       tty->ignore_num_lock = TRUE;
     tty->key_states |= GTK_TTY_STATE_NUM_LOCK;
     break;
-
+    
   case	GDK_Scroll_Lock:
     if (!(old_key_states & GTK_TTY_STATE_SCROLL_LOCK))
       tty->ignore_scroll_lock = TRUE;
     tty->key_states |= GTK_TTY_STATE_SCROLL_LOCK;
     break;
-
+    
   case	GDK_Shift_L:
   case	GDK_Shift_R:
     tty->key_states |= GTK_TTY_STATE_SHIFT;
     break;
-
+    
   case	GDK_Shift_Lock:
   case	GDK_Caps_Lock:
     if (!(old_key_states & GTK_TTY_STATE_CAPS_LOCK))
       tty->ignore_caps_lock = TRUE;
     tty->key_states |= GTK_TTY_STATE_CAPS_LOCK;
     break;
-
+    
   case	GDK_Control_L:
   case	GDK_Control_R:
     tty->key_states |= GTK_TTY_STATE_CONTROL;
     break;
-
+    
   case	GDK_Alt_L:
   case	GDK_Alt_R:
   case	GDK_Meta_L:
   case	GDK_Meta_R:
     tty->key_states |= GTK_TTY_STATE_ALT;
     break;
-
+    
   case	GDK_Super_L:
   case	GDK_Super_R:
     tty->key_states |= GTK_TTY_STATE_SUPER;
     break;
-
+    
   case	GDK_Hyper_L:
   case	GDK_Hyper_R:
     tty->key_states |= GTK_TTY_STATE_HYPER;
     break;
-
+    
   case	GDK_Mode_switch:
   case	GDK_Multi_key:
     break;
-
+    
   default:
     count = sprintf (buffer, "[%p]", (gpointer) event->keyval);
     break;
   }
-
-  gtk_tty_put_in (tty, buffer, count);
-
+  
+  buffer[count] = 0;
+  
   if (old_key_states != tty->key_states && !tty->freeze_leds)
     gtk_tty_leds_changed (tty);
-
+  
+  return_val = FALSE;
+  gtk_signal_emit (GTK_OBJECT (tty),
+		   tty_signals[KEY_PRESS],
+		   buffer,
+		   count,
+		   event->keyval,
+		   event->state,
+		   &return_val);
+  
+  if (!return_val)
+    gtk_tty_put_in (tty, buffer, count);
+  
   return TRUE;
 }
 
@@ -565,9 +614,9 @@ gtk_tty_key_release_event (GtkWidget	  *widget,
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TTY (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
-
+  
   tty = GTK_TTY (widget);
-
+  
   /* ok, at this point we still have the old state settings in event->state
    */
   old_key_states = tty->key_states;
@@ -585,7 +634,7 @@ gtk_tty_key_release_event (GtkWidget	  *widget,
     else
       tty->key_states &= ~GTK_TTY_STATE_NUM_LOCK;
     break;
-
+    
   case	GDK_Scroll_Lock:
     if (tty->ignore_scroll_lock)
       tty->ignore_scroll_lock = FALSE;
@@ -628,17 +677,17 @@ gtk_tty_key_release_event (GtkWidget	  *widget,
     tty->key_states &= ~GTK_TTY_STATE_HYPER;
     break;
   }
-
+  
   if (tty->key_pad_enter && !(tty->key_states & GTK_TTY_STATE_ALT))
   {
     tty->key_pad_enter = FALSE;
     gtk_tty_put_in (tty, &tty->key_pad_char, 1);
     tty->key_pad_char = 0;
   }
-
+  
   if (old_key_states != tty->key_states && !tty->freeze_leds)
     gtk_tty_leds_changed (tty);
-
+  
   return TRUE;
 }
 
@@ -648,15 +697,15 @@ gtk_tty_text_resize		(GtkTerm	*term,
 				 guint		*new_height)
 {
   GtkTty *tty;
-
+  
   g_return_if_fail (term != NULL);
   g_return_if_fail (GTK_IS_TTY (term));
-
+  
   tty = GTK_TTY (term);
-
+  
   if (parent_class->text_resize)
     (* parent_class->text_resize) (term, new_width, new_height);
-
+  
   if (tty->pty_fd >= 0)
   {
     gint	err;
@@ -681,19 +730,19 @@ gtk_tty_destroy (GtkObject *object)
   g_return_if_fail (GTK_IS_TTY (object));
   
   tty = GTK_TTY (object);
-
+  
   gtk_tty_set_emu (tty, NULL, NULL, NULL, NULL);
-
+  
   gtk_tty_os_close_pty (tty);
-
+  
   list = tty->update_leds;
   while (list)
   {
     GtkTtyUpdateLed *update_led;
-
+    
     update_led = list->data;
     list = list->next;
-
+    
     gtk_object_unref (GTK_OBJECT (update_led->led));
     g_free (update_led);
   }
@@ -701,46 +750,29 @@ gtk_tty_destroy (GtkObject *object)
   
   g_free (tty->prg_name);
   tty->prg_name = NULL;
-
+  
   if (GTK_OBJECT_CLASS (parent_class)->destroy)
     (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 static void
-gtk_tty_input_func (gpointer		data,
-		    gint		source,
-		    GdkInputCondition	condition)
+gtk_tty_marshal_signal_key_press (GtkObject		*object,
+				  GtkSignalFunc	func,
+				  gpointer		func_data,
+				  GtkArg		*args)
 {
-  GtkTty	*tty;
-  gint		len;
-  gchar		buffer[INPUT_BUFFER_LEN];
-
-  g_return_if_fail (data != NULL);
-  g_return_if_fail (GTK_IS_TTY (data));
-
-  tty = data;
-
-  len = read (tty->pty_fd, buffer, INPUT_BUFFER_LEN);
-
-  if (len > 0)
-    gtk_tty_put_out (tty, buffer, len);
-  else if (len < 0)
-  {
-    /* we assume the child died....
-     */
-    /* g_warning ("read(\"%s\",,) failed: %s", tty->tty_name, g_strerror (errno)); */
-
-    gdk_input_remove (tty->input_tag);
-    tty->input_tag = 0;
-
-    gtk_tty_os_wait (tty);
-
-    gtk_signal_emit (GTK_OBJECT (tty),
-		     tty_signals[PROGRAM_EXIT],
-		     tty->prg_name,
-		     tty->exit_status,
-		     tty->exit_signal);
-  }
+  GtkTtySignalKeyPress rfunc;
+  gint *return_val;
+  
+  rfunc = (GtkTtySignalKeyPress) func;
+  return_val = GTK_RETLOC_BOOL (args[4]);
+  
+  *return_val = (* rfunc) (object,
+			   GTK_VALUE_STRING (args[0]),
+			   GTK_VALUE_UINT (args[1]),
+			   GTK_VALUE_UINT (args[2]),
+			   GTK_VALUE_UINT (args[3]),
+			   func_data);
 }
 
 static void
@@ -750,9 +782,9 @@ gtk_tty_marshal_signal_exec (GtkObject		*object,
 			     GtkArg		*args)
 {
   GtkTtySignalExec rfunc;
-
+  
   rfunc = (GtkTtySignalExec) func;
-
+  
   (* rfunc) (object,
 	     GTK_VALUE_STRING (args[0]),
 	     GTK_VALUE_POINTER (args[1]),
@@ -767,9 +799,9 @@ gtk_tty_marshal_signal_exit (GtkObject		*object,
 			     GtkArg		*args)
 {
   GtkTtySignalExit rfunc;
-
+  
   rfunc = (GtkTtySignalExit) func;
-
+  
   (* rfunc) (object,
 	     GTK_VALUE_STRING (args[0]),
 	     GTK_VALUE_CHAR (args[1]),
@@ -788,16 +820,16 @@ gtk_tty_add_update_led (GtkTty		   *tty,
   g_return_if_fail (GTK_IS_TTY (tty));
   g_return_if_fail (led != NULL);
   g_return_if_fail (GTK_IS_LED (led));
-
+  
   update_led = g_new (GtkTtyUpdateLed, 1);
-
+  
   update_led->led = led;
   update_led->mask = mask;
-
+  
   gtk_object_ref (GTK_OBJECT (update_led->led));
-
+  
   tty->update_leds = g_list_append (tty->update_leds, update_led);
-
+  
   gtk_led_set_state (update_led->led,
 		     GTK_STATE_SELECTED,
 		     tty->leds & update_led->mask);
@@ -807,27 +839,65 @@ void
 gtk_tty_leds_changed (GtkTty	     *tty)
 {
   GList *list;
-
+  
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
-
+  
   if (!tty->freeze_leds)
     tty->leds = tty->key_states;
-
+  
   /* FIXME: invoke signal handler */
-
+  
   list = tty->update_leds;
   while (list)
   {
     GtkTtyUpdateLed *update_led;
-
+    
     update_led = list->data;
     list = list->next;
-
+    
     gtk_led_switch (update_led->led, tty->leds & update_led->mask);
   }
 }
 
+static void
+gtk_tty_input_func (gpointer		data,
+		    gint		source,
+		    GdkInputCondition	condition)
+{
+  GtkTty	*tty;
+  gint		len;
+  gchar		buffer[INPUT_BUFFER_LEN];
+  
+  g_return_if_fail (data != NULL);
+  g_return_if_fail (GTK_IS_TTY (data));
+  
+  tty = data;
+  
+  len = read (tty->pty_fd, buffer, INPUT_BUFFER_LEN);
+  
+  if (len > 0)
+    gtk_tty_put_out (tty, buffer, len);
+  else if (len < 0)
+  {
+    /* we assume the child died....
+     */
+    
+    gdk_input_remove (tty->input_tag);
+    tty->input_tag = 0;
+    
+    gtk_tty_os_wait (tty);
+    
+    gtk_signal_emit (GTK_OBJECT (tty),
+		     tty_signals[PROGRAM_EXIT],
+		     tty->prg_name,
+		     tty->exit_status,
+		     tty->exit_signal);
+  }
+  else
+    g_warning ("hum, zero characters to read on input ready for read?");
+}
+
 void
 gtk_tty_put_out (GtkTty		*tty,
 		 guchar		*buffer,
@@ -835,7 +905,7 @@ gtk_tty_put_out (GtkTty		*tty,
 {
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
-
+  
   if (tty->emu_func)
     while (count)
     {
@@ -850,7 +920,7 @@ gtk_tty_put_in (GtkTty	       *tty,
 {
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
-
+  
   if (tty->pty_fd < 0 || !tty->pid)
     gtk_tty_put_out (tty, buffer, count);
   else
@@ -866,10 +936,10 @@ gtk_tty_set_emu (GtkTty		 *tty,
 {
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
-
+  
   if (tty->emu_data && tty->emu_free_func)
     (*tty->emu_free_func) (tty->emu_data);
-
+  
   if (tty->emu_term)
   {
     g_free (tty->emu_term);
@@ -893,14 +963,14 @@ gtk_tty_execute (GtkTty		*tty,
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
   g_return_if_fail (tty->pid == 0);
-
+  
   tty->exit_status = 0;
   tty->exit_signal = 0;
-
+  
   
   g_free (tty->prg_name);
   tty->prg_name = NULL;
-
+  
   if (!strchr (prg_name, '/'))
   {
     gchar *p;
@@ -945,7 +1015,7 @@ gtk_tty_execute (GtkTty		*tty,
   if (!tty->prg_name)
     tty->prg_name = g_strdup (prg_name);
   
-
+  
   if (tty->emu_reset_func)
   {
     g_free (tty->emu_term);
@@ -955,9 +1025,9 @@ gtk_tty_execute (GtkTty		*tty,
   if (!tty->tty_name)
   {
     g_assert (tty->pty_fd == -1); /* paranoid */
-
+    
     gtk_tty_os_open_pty (tty);
-
+    
     if (!tty->tty_name)
     {
       g_warning ("could not find a free pseudo teletype");
@@ -972,7 +1042,7 @@ gtk_tty_execute (GtkTty		*tty,
     gtk_tty_os_close_pty (tty);
     return ;
   }
-    
+  
   tty->pid = fork ();
   if (tty->pid < 0)
   {
@@ -980,7 +1050,7 @@ gtk_tty_execute (GtkTty		*tty,
     fclose (f_tty);
     return ;
   }
-
+  
   if (tty->pid == 0) /* child process */
   {
     if (envp == environ)
@@ -992,16 +1062,16 @@ gtk_tty_execute (GtkTty		*tty,
       else
       {
 	gchar *buffer;
-
+	
 	buffer = g_new (gchar, strlen (empty_term) + strlen (tty->emu_term) + 1);
 	buffer[0] = 0;
 	strcat (buffer, empty_term);
 	strcat (buffer, tty->emu_term);
 
 	if (putenv (buffer) < 0)
+	{
 	  putenv (empty_term);
-
-	g_free (buffer);
+	}
       }
 
       envp = environ;
@@ -1009,19 +1079,19 @@ gtk_tty_execute (GtkTty		*tty,
     
     /* this function will _exit(-1) on errors*/
     gtk_tty_os_setup_tty (tty, f_tty, GTK_TTY_CLASS (GTK_OBJECT (tty)->klass)->os_hintp);
-
+    
     execve (tty->prg_name, argv, envp);
-
+    
     fprintf (stderr, "gtktty-clone: %s: %s\n", tty->prg_name, g_strerror (errno));
     _exit (-1);
   }
-
+  
   /* parent process */
-
+  
   fclose (f_tty);
-
+  
   tty->input_tag = gdk_input_add (tty->pty_fd, GDK_INPUT_READ, gtk_tty_input_func, tty);
-
+  
   gtk_signal_emit (GTK_OBJECT (tty),
 		   tty_signals[PROGRAM_EXEC],
 		   tty->prg_name,
@@ -1037,7 +1107,7 @@ gtk_tty_exec (GtkTty	     *tty,
 {
   g_return_if_fail (tty != NULL);
   g_return_if_fail (GTK_IS_TTY (tty));
-
+  
   /* FIXME: call gtk_tty_execute() */
 }
 
